package com.cloudlink.open.util;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.cloudlink.open.common.Constant;
import com.cloudlink.open.common.User;
import com.cloudlink.open.config.WeDemoConfig;
import com.google.gson.Gson;

@Component("freeAccessHelper")
public class FreeAccessHelper {
	private static final Logger logger = LoggerFactory.getLogger(FreeAccessHelper.class);
	@Autowired
	private WeDemoConfig weDemoConfig;
	private static RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(5000) // 设置连接超时时间
			.setConnectionRequestTimeout(5000) // 设置请求超时时间
			.setSocketTimeout(5000).setRedirectsEnabled(true)// 默认允许自动重定向
			.build();

	/**
	 * 通过code获取CloudLink用户详情
	 * @param code
	 * @return
	 */
	public String getUserId(String code) {
		String userId = null;
		CloseableHttpClient httpClient = HttpClients.createDefault();
		try {
			//首先获取CloudLink token
			String token = getAccessToken(httpClient, code);
			if (token != null) {
				//获取CloudLink中用户id
				userId = getUserId(httpClient, token);
			}

		} catch (Exception e) {
			logger.info("Http request url: exception" + e.getMessage());
		} finally {
			try {
				if (httpClient != null) {
					httpClient.close();
				}

			} catch (IOException e) {
				logger.info("close httpclient error", e);
			}

		}
		return userId;

	}

	/**
	 * 通过code获取CloudLink用户详情
	 * @param code
	 * @return
	 */
	public User getUser(String code) {
		User user = null;
		String userId = null;

		CloseableHttpClient httpClient = HttpClients.createDefault();
		try {

			//首先获取CloudLink token
			String token = getAccessToken(httpClient, code);
			if (token != null) {
				//其次，获取CloudLink中用户id
				userId = getUserId(httpClient, token);
				//获取CloudLink中用户详情
				user = getUserInfo(httpClient, token, userId);
			}

		} catch (Exception e) {
			logger.info("Http request url: exception" + e.getMessage());
		} finally {
			try {
				if (httpClient != null) {
					httpClient.close();
				}

			} catch (IOException e) {
				logger.info("close httpclient error", e);
			}

		}
		return user;

	}

	/**
	 * 通过code、appid、secret获了CloudLink 的Token
	 * @param httpClient
	 * @param code
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	private String getAccessToken(CloseableHttpClient httpClient, String code)
			throws ClientProtocolException, IOException {
		String token = null;
		CloseableHttpResponse response = null;
		// 创建Http Post请求
		HttpPost httpPost = new HttpPost(weDemoConfig.getTokenURL());
		Map<String, String> bodyMap = new HashMap<String, String>();
		bodyMap.put(Constant.CLIENT_ID_KEY, weDemoConfig.getAppId());
		bodyMap.put(Constant.CLIENT_SECRET_KEY, weDemoConfig.getAppSecret());
		bodyMap.put(Constant.CODE_KEY, code);
		bodyMap.put(Constant.TYPE_KEY, "u");
		StringEntity entity = new StringEntity(new Gson().toJson(bodyMap), "UTF-8");
		httpPost.addHeader(HTTP.CONTENT_TYPE, Constant.APPLICATION_JSON_KEY);
		httpPost.setConfig(requestConfig);
		httpPost.setEntity(entity);
		response = httpClient.execute(httpPost);
		if (response.getStatusLine().getStatusCode() != 200) {
			logger.info("Http request url: fail" + EntityUtils.toString(response.getEntity(), "utf-8"));
		} else {
			String resultString = EntityUtils.toString(response.getEntity(), "utf-8");

			Map<String, String> map = new Gson().fromJson(resultString, Map.class);
			if (map != null && map.get(Constant.ACCESS_TOKEN_KEY) != null) {
				token = (String) map.get(Constant.ACCESS_TOKEN_KEY);
			}
		}
		return token;
	}

	/**
	 * 通过token获取CloudLink中用户ID
	 * @param httpClient
	 * @param token
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	private String getUserId(CloseableHttpClient httpClient, String token) throws ClientProtocolException, IOException {
		String userId = null;
		CloseableHttpResponse response = null;
		logger.info("Http request url:" + weDemoConfig.getUserIdURL() + ",token:" + token);
		HttpGet httpGet = new HttpGet(weDemoConfig.getUserIdURL());
		httpGet.addHeader(Constant.AUTHORIZATION_KEY, token);
		httpGet.setConfig(requestConfig);
		response = httpClient.execute(httpGet);
		if (response.getStatusLine().getStatusCode() != 200) {
			logger.info("Http request url: fail" + EntityUtils.toString(response.getEntity(), "utf-8"));
		} else {
			String resultString = EntityUtils.toString(response.getEntity(), "utf-8");

			Map<String, String> map = new Gson().fromJson(resultString, Map.class);
			if (map != null && map.get(Constant.USERID_KEY) != null) {
				userId = (String) map.get(Constant.USERID_KEY);
				logger.info("get userid " + userId);
			}
		}
		return userId;
	}

	/**
	 * 通过token获取CloudLink中用户详情
	 * @param httpClient
	 * @param token
	 * @param userId
	 * @return
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	private User getUserInfo(CloseableHttpClient httpClient, String token, String userId)
			throws ClientProtocolException, IOException {
		User user = null;
		CloseableHttpResponse response = null;

		String url = weDemoConfig.getUserInfoURL() + "?userId=" + userId;
		logger.info("Http request url:" + url);
		HttpGet httpGet = new HttpGet(url);
		httpGet.addHeader(Constant.AUTHORIZATION_KEY, token);
		httpGet.setConfig(requestConfig);
		response = httpClient.execute(httpGet);
		if (response.getStatusLine().getStatusCode() != 200) {
			logger.info("Http request url: fail" + EntityUtils.toString(response.getEntity(), "utf-8"));
		} else {
			String resultString = EntityUtils.toString(response.getEntity(), "utf-8");

			Map<String, String> map = new Gson().fromJson(resultString, Map.class);
			if (map != null && map.get(Constant.USERNAMECN_KEY) != null) {
				String userName = (String) map.get(Constant.USERNAMECN_KEY);
				logger.info("get userName " + userName);
				user = new User();
				user.setUserId(userId);
				user.setUserName(userName);
			}
		}
		return user;
	}
}
